; USE "SCRIPTS PLAY" TO PSEUDO-COMPILE LIBRARY AND START DEMONSTRATION.
;
; INADEQUATELY DOCUMENTED CODE IS INCLUDED.
;
; The following procedures constitute a HELP system which I am developing for a
; new app.  The general concept is to use the portion of form "F" from 2,0 to
; 24,45 for data entry.  The portion from 2,47 to 24,79 displays standard help
; information such as [F1], [F2], [F7], [Ctrl Z], etc.  Don't be fooled by the
; fact that the Paradox Form Editor counts Column 0 as 1.  It is still 0.
;
; My prime motivation for writing this system is the "Array Help."  My
; application would normally require at least 16 HelpAndFill lookup tables which
; creates a number of problems related to both memory management and speed.
;
; This system is intended for use in CoEdit mode on stand-alone (non-networked)
; 286+s.  If you desire to use it in another configuration changes may be
; required.  In fact, I have no doubt that two weeks from now I too will make
; substantial changes based on discovering various "bugs."  If you find some,
; please send me a note at CompuServe [76164,2640] or contact me at:
;
;                        Daniel A. Morgan
;                        Morgan Information Systems, Inc.
;                        Post Office Box 505
;                        Bellevue, WA 98009-9505
;                        (206) 947-3626
;
; Improvements and suggestions, not involving sharply pointed objects, are
; appreciated.
;
; Any use of this code in part, or in whole, in a commercial application
; requires an acknowlegement of its source.  Thank-you.
;
; When the app's user presses [F1] for help one of four types of help is
; provided.  Three types of help are handled by the entry in the HelpSwitch
; proc.  They are as follows:
;
; 1.  Text:  Text help overwrites the standard help information on Form "F" with
;     text information specifically related the field in which the cursor is
;     located.
;
; 2.  Array:  Array help is a simulation, and combination, of what I consider
;     to be the strengths of Paradox's HelpAndFill and ValCheck picture systems.
;     The only substantial change I made was to replace the HelpAndFill [F2] key
;     (which I strongly dislike) with the [Enter] key.  For this system to work
;     properly, the items in the lookup array MUST be arranged in alphabetical
;     order.
;
; 3.  HelpAndFill:  The proc "HelpKey" activates Paradox's standard HelpAndFill.
;
; The fourth type of help is function related help.  This help, not demonstrated
; here, shows a help screen specific to "ZOOM" if the user presses [Ctrl Z]
; followed by [F1] or specific to "FIELD VIEW" if the user presses [Ctrl F]
; or [Alt F5] followed by [F1], etc.  The App's main menu also has a special
; help menu.
;
; The following code will write itself into a Paradox library named "HelpLib."
; My Thanks to Daniel J. Paolini II - dp Solutions, Inc., and others, for the
; idea of displaying ASCII character graphics during pseudo-compilation.
;
;
;
;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;|                             BEGIN CODE SECTION                            ||
;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||


Cursor OFF
CREATELIB "HelpLib"
@6,2 ?? "COMPILING HELPLIB LIBRARY"
PaintCanvas Attribute 79 6,1,6,27
PaintCanvas Fill CHR(07) 6,29,6,38
;=============================================================================
PROC HelpSwitch()            ; HelpSwitch checks the field in which the cursor
                             ; is located and determines which help proc to call
Release Vars LUHELP          ; Release previous help array if in memory

SWITCH
     CASE Field() = "CLIENT #"
       OR Field() = "MASTER #"  : Client()           ; An example of Text help
     CASE Field() = "FIRST NAME"
       OR Field() = "LAST NAME" : MESSAGE " NO HELP AVAILABLE - Press Any Key "
                                  dump = GetChar() Release Vars dump
     CASE Field() = "SEX"       : Sex()              ; An example of Text help
     CASE Field() = "ETHNICITY" : Ethnicity()        ; An example of Paradox's
     CASE Field() = "STAFF"     : HelpKey()          ; HelpAndFill help
     OTHERWISE : Return False
ENDSWITCH

If IsAssigned(LUHELP[1]) Then           ; If Array help has been called the
     LU_Engine()                        ; first element of the array will
EndIf                                   ; always be assigned.  If assigned, call
                                        ; the Array Help main proc LU_Engine.
ENDPROC
Writelib "HelpLib" HelpSwitch
Release Procs HelpSwitch
@6,29 ?? "\004"
;=========================================================== LU_Engine()
PROC LU_Engine()             ; This procedure displays the Help Array on
                             ; help portion of the form and calls

PRIVATE i,                   ; FOR loop counter
        ASize.s              ; Size of help array

Canvas OFF                   ; Freeze the canvas
Cursor OFF                   ; Stop the cursor from blinking

@2,48 Clear EOL              ; Clear the forms HELP header
STYLE Attribute 14           ; Yellow on Black text
@2,48 ?? Header.a            ; Display the field specific header

STYLE Attribute 127          ; White on Gray text
PaintCanvas Fill CHR(255) Attribute 127 3,47,24,79 ; Overwrite form's help text
                                                   ; with spaces (ASCII 255)
ASize.s = ArraySize(LUHELP)                        ; Size of the help array
FOR i FROM 1 TO ASize.s                            ; For each item in the array
     @i+2,48 ?? LUHELP[i]                          ; Display it beginning on
ENDFOR                                             ; line 2 of the form (i+2)

Canvas ON                                          ; Update the canvas with the
STYLE                                              ; HELP Array
LU_Search()                                        ; Call next proc

If Retval Then                                     ; If search was successful
     Enter                                         ; move to the next field on
EndIf                                              ; the form

ENDPROC
Writelib "HelpLib" LU_Engine
Release Procs LU_Engine
@6,30 ?? "\004"
;=========================================================== LU_Search()
PROC LU_Search()           ; This procedure searches each item in the
                           ; HELP array, calls the proc LUHiLite to highlight
                           ; the first example matching the input string
                           ; and types the choosen selection into the
                           ; field when the user uses movement keys to
                           ; highlight the correct selection and presses
                           ; [Enter] (NOT [F2]) or the entered search string
                           ; reaches a point where it has narrowed the choice
                           ; to a single entry in the HELP array

PRIVATE Cur_char.s,        ; # of characters in search string
        Cur_choice.s,      ; Currently highlighted lookup help screen row
        i,                 ; FOR loop variable
        Next_choice.s,     ; Next screen row to highlight
        Srch_string.a,     ; Search string
        x                  ; String search expansion counter

Cur_char.s = 1             ; Initialize search with first character
Cur_choice.s = 3           ; Initialize at screen row 3
Next_choice.s = 3          ; Initialize at screen row 3
Srch_string.a = ""         ; Initialize a blank string

While True                 ; Top of closed loop

LUHiLite()                 ; Highlight the default or choosen array element
                           ; at the beginning of each loop
Keychoice.s = GetChar()    ; Get a key from the user

SWITCH                     ; Evaluate the key pressed by the user

     CASE Keychoice.s = -80 : If Cur_choice.s - 2 < ASize.s Then    ; Down key
                                   Next_choice.s = Cur_choice.s+1   ; unless at
                              Else                                  ; bottom
                                   BEEP                             ; move down
                              EndIf                                 ; one

     CASE Keychoice.s = -79 : Next_choice.s = ASize.s+2             ; End key
                              Srch_string.a = ""                    ; Move to
                                                                    ; bottom

     CASE Keychoice.s = -72 : If Cur_Choice.s > 3 Then              ; Up key
                                   Next_choice.s = Cur_choice.s - 1 ; unless at
                              Else                                  ; top  move
                                   BEEP                             ; up one
                              EndIf

     CASE Keychoice.s = -71 : Next_choice.s = 3                     ; Home key
                              Srch_string.a = ""                    ; Move to
                                                                    ; top

     CASE Keychoice.s =   8 : Srch_string.a =            ; Backspace key
                                                         ; Shorter the search
                                                         ; string by 1 character
                              SUBSTR(Srch_string.a,1,(LEN(Srch_string.a)-1))

     CASE Keychoice.s =  13 :                          ; Enter key
          CtrlBackspace                                ; Clear the current field
          Typein LUHELP[Cur_choice.s-2]                ; Typein the selection
          QuitLoop

     CASE Keychoice.s =  27 : Return False                          ; Esc

     CASE Keychoice.s >= 32                                         ; Any letter
      AND Keychoice.s <=122 :
          Srch_string.a = Srch_string.a + CHR(Keychoice.s) ; Add next letter
          Cur_char.s = LEN(Srch_string.a)                  ; Get string's length
          FOR i FROM 1 TO ASize.s                          ; Look for a match
               If SUBSTR(LUHelp[i],1,Cur_char.s) =
                  SUBSTR(Srch_string.a,1,Cur_char.s) Then  ; If found
                    Next_choice.s = i+2                    ; HiLite it
                    QuitLoop                               ; Otherwise
              EndIf                                        ; do nothing
          ENDFOR
          SWITCH
               CASE i > ASize.s :                          ; The search failed
                    BEEP
                    Srch_string.a = SUBSTR(Srch_string.a,1,Cur_char.s-1)
                    Loop
               CASE i+1 > ASize.s :                        ; Last item in list
                    CtrlBackspace
                    Typein LUHelp[ASize.s]
                    Return True
               CASE SUBSTR(Srch_string.a,1,Cur_char.s) <>  ; This is the only
                    SUBSTR(LUHelp[i+1],1,Cur_char.s) :     ; item that can
                    CtrlBackspace                          ; possibly match
                    Typein LUHelp[i]
                    Return True
               OTHERWISE :                                 ; Automatically add
                    Srch_string.a = ""                     ; to the search
                    x = 1                                  ; string if other
                    While True                             ; similar entries
                         If SUBSTR(LUHELP[i],1,x) = SUBSTR(LUHELP[i+1],1,x) Then
                              Srch_string.a = Srch_string.a +
                              SUBSTR(LUHELP[i],x,1)
                              x = x + 1                    ; Check next letter
                         Else                              ; Search string
                              QuitLoop                     ; expanded as much
                         EndIf                             ; as possible
                    EndWhile
          ENDSWITCH
     CASE Keychoice.s = 127 : Srch_string.a = ""           ; CtrlBackspace key
                                                           ; Blank search string
     OTHERWISE : BEEP                                      ; Any other keys
ENDSWITCH

EndWhile                   ; Bottom of closed loop

Return True

ENDPROC
Writelib "HelpLib" LU_Search
Release Procs LU_Search
@6,31 ?? "\004"
;=========================================================== LUHiLite()
PROC LUHiLite()            ; This procedure is called at the top of the
                           ; While loop by LU_Search to highlight the best
                           ; match to the user's selection

                           ; First it returns to currently highighted selection
                           ; to the default help menu colors White on Gray
PaintCanvas Attribute 127 Cur_choice.s,47,Cur_choice.s,79
                           ; Then highlights the new selection White on Red
                           ; If you do not have a color monitor you should try
                           ; changing these choices to normal and reverse text
PaintCanvas Attribute  79 Next_choice.s,47,Next_choice.s,79
                           ; Reset Cur_choice.s variable equal to the last
                           ; choice made by the user
Cur_choice.s = Next_choice.s
                           ; These next three lines change the default text
                           ; to White on Red and display the current search
                           ; string on the bottom of the screen so the user
                           ; can see the criteria they have choosen.
STYLE ATTRIBUTE 79
@24,47 ?? " Search Key: " + Srch_string.a + "  "
STYLE                      ; Return default to White on Blue (at least for me)

ENDPROC
Writelib "HelpLib" LUHiLite
Release Procs LUHiLite
@6,32 ?? "\004"
;============================================================ Help_Setup()
PROC Help_Setup()          ; This proc is called by Text help to take care
                           ; of clearing the help text area and displays
                           ; the "PAK" (Press Any Key) message

Cursor OFF                                           ; Turn off the cursor
PaintCanvas Fill Chr(255) Attribute 127 2,47,24,79   ; Blank the help area
STYLE Attribute 14                                   ; Yellow on Black
@24,47 ?? "    PRESS ANY KEY TO CONTINUE    "        ; Message to user
STYLE                                                ; Return to default
SETMARGIN 48                                         ; Reset left margin
                                                     ; for text display
ENDPROC
Writelib "HelpLib" Help_Setup
Release Procs Help_Setup
@6,33 ?? "\004"
;============================================================ Client()
PROC Client()              ; An example of text help

Help_Setup()
Canvas OFF
@2,0                                               ; Begin help text
Text
The client number is automatic-
ally inserted by the computer
each time a new record is
created.

Each household receives a
master number and each of
the households members is
identified by a letter.

For example, the first intake
in 1993 will be 930001a. "93"
for the year (1993). The next
four digits "0001" is the
sequence number. The letter
"a" indicates that this is the
first person entered, the head
of household. Each subsequent
family member receives a letter
code, "b" through "z" as they
are entered into the database.
EndText
PaintCanvas Attribute 127 2,47,23,79               ; Color Bright White on White
Canvas ON
dump = GetChar()                                   ; Wait for user input
SETMARGIN 0                                        ; Reset margin

ENDPROC
Writelib "HelpLib" Client
Release Procs Client
@6,34 ?? "\004"
;============================================================ Ethnicity()
PROC Ethnicity()           ; An example of array help

                           ; These ethnic groups are used by the County of
                           ; King, State of Washington, and do not in any
                           ; manner constitutes the authors choices.  Except
                           ; perhaps for one of them.

; Note that if the letter "C" is the first entered for the search string that
; Cambodian (the first entry starting with "C" is highlighted.  If an "h" is
; entered next the highlight moves to Chicano/Latino but the search string is
; "Chi" since both the highlighted choice and the next choice, "Chinese" have
; these three letters in common.  Pressing a "c" or "n" narrows the choice to
; a single array element and types it into the form.  Starting the search
; string with "O" causes the search string to read "Other" and pressing the
; [Enter] key enters that choice or pressing the space bar identifies the
; user's choice as "Other Asian"

Header.a = "ETHNICITY"                            ; Displays on line 2
ARRAY LUHELP[17]                                  ; Declare HELP array
LUHELP[1] = "African American"
LUHELP[2] = "Alaskan Native"
LUHELP[3] = "Cambodian"
LUHELP[4] = "Chicano/Latino"
LUHELP[5] = "Chinese"
LUHELP[6] = "Filipino"
LUHELP[7] = "Japanese"
LUHELP[8] = "Korean"
LUHELP[9] = "Laotian"
LUHELP[10] = "Mixed Heritage"
LUHELP[11] = "Native American/Alaskan"
LUHELP[12] = "Other"
LUHELP[13] = "Other Asian"
LUHELP[14] = "PAL Programmer"
LUHELP[15] = "Pacific Islander"
LUHELP[16] = "Vietnamese"
LUHELP[17] = "White"

ENDPROC
Writelib "HelpLib" Ethnicity
Release Procs Ethnicity
@6,35 ?? "\004"
;=========================================================== HelpKey()
PROC HelpKey()             ; An example using Paradox's HelpAndFill help

Echo Normal

KeyPress wait_key
While (HelpMode() = "LookupHelp")
     KeyPress GetChar()
EndWhile

Echo Off
Enter

ENDPROC
Writelib "HelpLib" HelpKey
Release Procs HelpKey
@6,36 ?? "\004"
;=========================================================== Sex()
PROC Sex()                 ; A text help example which concatenates
                           ; a first and last name and displays them
                           ; as part of the help text.
PRIVATE x,     ; Client's first name
        y,     ; Client's last name
        z,     ; Number of blank spaces left in help text area less name
        a,     ; Offset from column 47 for displaying name
        b      ; Column in which to begin display of name

Help_Setup()

x = [First Name]                 ; From first name field
y = [Last Name]                  ; From last name field
z = 32 - (LEN(x + y) + 1)        ; Help area is 32 spaces wide
                                 ; Get length of first & last names plus the
                                 ; space separating them (the 1)
a = INT(z/2)                     ; divide by 2 and don't allow fractions

b = 47 + a                       ; Add offset integrer to display column 47

Canvas OFF
@2,0                             ; At row 2 start text.  Left margin was
                                 ; set to column 48 by Help_Setup proc
Text

Enter the sex of:
EndText

@4,b ?? " " + STRVAL(x) + " " + STRVAL(y) + " "     ; display the name

@5,0                                                ; more text
Text
"F" for female, or "M" for male.
More creative choices are not
allowed.

The "F/M" choice is enforced by
ValCheck Picture.
EndText

PaintCanvas Attribute 127 2,47,23,79
Canvas ON
dump = GetChar()
SETMARGIN 0

ENDPROC
Writelib "HelpLib" Sex
Release Procs Sex
@6,37 ?? "\004"
;=========================================================== Wait_Master()
PROC Wait_Master()        ; A very simplified (totally brain dead)
                          ; Wait Table procedure

Reset Clear

CoEdit "Fam_Mem"
PICKFORM "F"

While True
     Wait Table
          Prompt " HELP SYSTEM DEMONSTRATION",
                 " Press [F1] For Help!"
          Until
          - 60 ,      ; [F2]
          - 59        ; [F1]

          wait_key = retval

          SWITCH
               CASE wait_key = - 59 : HelpSwitch()
               CASE wait_key = - 60 : Do_It! Reset Quit
          ENDSWITCH
EndWhile

ENDPROC
Writelib "HelpLib" Wait_Master
Release Procs Wait_Master
@6,38 ?? "\004"
;=============================================================== End of Procs
SLEEP 200
Reset Clear
Release Procs All
Release Vars All
SETSWAP 25000
AUTOLIB = "HELPLIB"
Wait_Master()
Release Procs All
Release Vars All
